%--------------------------------------------------------------------------
% LOAD SOUND FILE
wavfile = 'pianotrump';
[music, Fs] = wavread(wavfile);
time = (length(music)-1)/Fs;

%--------------------------------------------------------------------------
% FIGURE: WAVEFORM
mfig('Waveform');
t_axis = linspace(0, time, length(music));
plot(t_axis, music);
xlabel('Time [s]');
xlim([0 time]);


%--------------------------------------------------------------------------
% COMPUTE SPECTROGRAM
nfft = 2048;
noverlap = nfft/2;
zeropad = 512;
C = mspecgramz(music(:,1),nfft,nfft-noverlap,zeropad);


%--------------------------------------------------------------------------
% FIGURE: SPECTROGRAM
mfig('Spectrogram');
t_axis = linspace(0, time, size(C,2));
f_axis = linspace(0, Fs/2, size(C,1));
imagesc(t_axis, f_axis, abs(C)); axis xy;
colormap(1-gray);
xlabel('Time [s]');
ylabel('Frequency [Hz]');


%--------------------------------------------------------------------------
% SPECTROGRAM WITH LOGARITHMIC FREQUENCY AXIS
[V,MX] = logfsgram(music, nfft, Fs, nfft, noverlap, 50, 24);


%--------------------------------------------------------------------------
% FIGURE: LOG. FRQ. SPECTROGRAM
mfig('Log. frq. spectrogram');
t_axis = linspace(0, time, size(V,2));
f_axis = 1:size(V,1);
imagesc(t_axis, f_axis, abs(V)); 
axis xy; 
colormap(1-gray);
xlabel('Time [s]');
ylabel('Frequency bin');


%--------------------------------------------------------------------------
% NON-NEGATIVE MATRIX FACTOR 2-D DECONVOLUTION
d = 2;
opts.Tau = 0:3;
opts.Phi = 0:31;
opts.maxiter = 200;
opts.costfcn = 'ls';
opts.lambda = 0;
opts.plotiter = 5;
opts.plotfcn = @plot_nmf2d;
opts.displaylevel = 'iter';
opts.conv_criteria = 1e-4;
[W,H]=isp_snmf2d(V,d,opts);


%--------------------------------------------------------------------------
% FIGURE: FINAL RECONSTRUCTION
R = isp_nmf2d_rec(W,H,opts.Tau,opts.Phi,1:d);
mfig('Final Reconstruction');
t_axis = linspace(0, time, size(R,2));
f_axis = 1:size(R,1);
imagesc(t_axis, f_axis, abs(R)); axis xy; 
colormap(1-gray);
xlabel('Time [s]');
ylabel('Frequency bin');


%--------------------------------------------------------------------------
% WAVEFORM RECONSTRUCTION BY SPECTROGRAM MASKING
R = [];
for idx = 1:d
    Rec = isp_nmf2d_rec(W, H, opts.Tau, opts.Phi, idx);
    R(:,:,idx) = sqrt(MX'*(Rec.^2));
end

for k = 1:size(R,3)
    mask = logical(R(:,:,k)==max(R,[],3));
    r = mispecgramm(C, mask, nfft-noverlap, zeropad);
    r = r./max(abs(r));
    wavwrite(r,Fs,sprintf([wavfile, '%d.wav'], k));
end


